/*
 * Copyright (C) 2019 CERN (www.cern.ch)
 * Author: Jean-Claude BAU - CERN
 *
 * Released according to the GNU LGPL, version 2.1 or any later version.
 *
 */

#include <time.h>

#include <libwr/wrs-msg.h>

#include "hal_timer.h"

#define TIMER_MAX_BLOCKS 5

static timer_parameter_t *blocks[TIMER_MAX_BLOCKS];
static time_t start, end;

static void _timer_register_block(timer_parameter_t *p) {
	int i;

	for (i=0; i<TIMER_MAX_BLOCKS; i++ ) {
		if ( blocks[i]==p) {
			// Already registered
			return;
		}
		if ( blocks[i]==NULL) {
			// Register the block
			blocks[i]=p;
			return;
		}
	}
}

void timer_init(timer_parameter_t *p) {
	_timer_register_block(p);
	while (p->id!=TMO_END_BLOCK) {
		p->tmoMsRef=p->tmoMs; // Save the original value
		libwr_tmo_init(&p->timer,p->tmoMs, p->repeat);
		p++;
	}
}


void timer_scan(timer_parameter_t *p) {

	while (p->id!=TMO_END_BLOCK) {
		if (libwr_tmo_expired(&p->timer) && p->cb!=NULL) {
			(*p->cb)(p->id);
		}
		p++;
	}
}

void timer_update_tmo(tmo_id_t id,uint32_t tmoMs) {
	int i;

	for (i=0; i<TIMER_MAX_BLOCKS; i++ ) {
		timer_parameter_t *p;

		if ( (p=blocks[i])!=NULL ) {
			while (p->id!=TMO_END_BLOCK) {
				if (p->id==id) {
					if ( p->tmoMs!=tmoMs ) {
						p->tmoMs=tmoMs;
						pr_info("%s: Timer %d set to %dms\n",__func__,id,p->tmoMs);
						libwr_tmo_init(&p->timer,p->tmoMs, p->repeat);
						 time(&start);
					}
				}
				p++;
			}
		}
	}
}

void timer_restore_tmo(tmo_id_t id) {
	int i;

	for (i=0; i<TIMER_MAX_BLOCKS; i++ ) {
		timer_parameter_t *p;

		if ( (p=blocks[i])!=NULL ) {
			while (p->id!=TMO_END_BLOCK) {
				if (p->id==id) {
					if ( p->tmoMs!=p->tmoMsRef) {

						p->tmoMs=p->tmoMsRef;
						pr_info("%s: Timer %d restored to %dms\n",__func__,id,p->tmoMs);
						libwr_tmo_init(&p->timer,p->tmoMs, p->repeat);
						time(&end);
						pr_info("%s: Execution time %f seconds\n",__func__,difftime(end, start));
					}
				}
				p++;
			}
		}
	}
}
